import time
from calendar import monthrange

from netforce.model import Model,fields,get_model
from netforce.access import get_active_company

from . import utils

class ReportStaffFeeDetail(Model):
    _name="clinic.report.staff.fee.detail"
    _string="Report Staff Fee Detail"
    _transient=True
    
    _fields={
        "date": fields.Date("Month"),
        "date_from": fields.Date("From", required=True),
        "date_to": fields.Date("To", required=True),
        'staff_id': fields.Many2One("clinic.staff","Staff"),
        "type": fields.Selection([["doctor","Doctor"],["nurse","Nurse"],["staff","Staff"]],"Type"),
        'department_id': fields.Many2One("clinic.department","Department"),
        'branch_id': fields.Many2One("clinic.branch","Branch"),
    }

    def _get_date_from(self,context={}):
        year,month=time.strftime("%Y-%m").split("-")
        return '%s-%s-01'%(year,month)

    def _get_date_to(self,context={}):
        year,month,day=time.strftime("%Y-%m-%d").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        return "%s-%s-%s"%(year,month,total_day)

    _defaults={
        'date': lambda *a: time.strftime("%Y-%m-%d"),
        'date_from': _get_date_from,
        'date_to': _get_date_to,
    }
    
    def get_report_data(self,ids,context={}):
        company_id=get_active_company()
        company=get_model('company').browse(company_id)
        year, month=time.strftime("%Y-%m").split("-")
        weekday, total_day=monthrange(int(year), int(month))
        time_start='%s-%s-01'%(year,str(month).zfill(2))
        time_stop='%s-%s-%s'%(year,str(month).zfill(2),total_day)

        defaults=context.get('defaults')
        if defaults:
            year,month,day=defaults['date'].split("-")
            time_start='%s-%s-01'%(year,str(month).zfill(2))
            time_stop='%s-%s-%s'%(year,str(month).zfill(2),total_day)
        staff_id=None
        staff_type=None
        department_id=None
        branch_id=None
        if ids:
            obj=self.browse(ids)[0]
            month=obj.date_from.split("-")[1]
            time_start=obj.date_from
            time_stop=obj.date_to
            staff_id=obj.staff_id.id
            staff_type=obj.type
            department_id=obj.department_id.id
            branch_id=obj.branch_id.id
        # new patient of this month
        dom=[]
        dom.append(['date','>=',time_start])
        dom.append(['date','<=',time_stop])
        if department_id:
            dom.append(['staff_id.department_id','=',department_id])
        if branch_id:
            dom.append(['staff_id.branch_id','=',branch_id])
        staff_name=''
        if staff_id:
            dom.append(['staff_id','=',staff_id])
        lines=[]
        total_qty=0
        total_amount=0
        lines=[]
        for lcline in get_model('clinic.labor.cost.line').search_browse(dom):
            lc=lcline.labor_cost_id
            item=lc.cycle_item_id
            date=item.date
            cycle=item.cycle_id
            branch=item.branch_id
            dpt=item.department_id
            items=[]
            for dline in lc.doctor_lines:
                staff=dline.staff_id
                amount=dline.amount or 0
                rate=dline.rate or 0
                items.append({
                    'staff_id': staff.id,
                    'type': 'doctor',
                    'rate': rate,
                    'amount': amount,
                })
            for nline in lc.nurse_lines:
                staff=nline.staff_id
                rate=dline.rate or 0
                items.append({
                    'staff_id': staff.id,
                    'type': 'nurse',
                    'rate': rate,
                    'amount': amount,
                })
            for item in items:
                print(item)
           
        month_str=utils.MONTHS['th_TH'][int(month)]
        start=int(time_start[8:10])
        stop=int(time_stop[8:10])
        diff=stop-start
        data={
            'company_name': company.name or "",
            'parent_company_name': company.parent_id.name or "",
            'staff_name': staff_name,
            'lines': sorted(lines,key=lambda x: x['date']),
            'total_qty': total_qty,
            'total_amount': total_amount,
            'month': month_str,
            'from': time_start,
            'to': time_stop,
            'is_duration': diff+1 < total_day,
            'year': year,
        }
        return data

    def onchange_date(self,context={}):
        data=context['data']
        date=data['date']
        year,month,day=date.split("-")
        weekday, total_day=monthrange(int(year), int(month))
        data['date_from']="%s-%s-01"%(year,month)
        data['date_to']="%s-%s-%s"%(year,month,total_day)
        return data

ReportStaffFeeDetail.register()
